// HSS.Forms.ModuleBase.cs
// ----------------------------------------------------------------------------
// Licensed under the MIT license
// http://www.opensource.org/licenses/mit-license.html
// ----------------------------------------------------------------------------
// HighSpeed-Solutions, LLC
// Copyright (c) 2001-2010
// ----------------------------------------------------------------------------
// File:       ModuleBase.cs
// Author:     HSS\gbanta
// Created:    08/12/2010
// Modified:   12/04/2010
// ----------------------------------------------------------------------------
namespace HSS.Forms
{
	#region Using Directives
	using System;
	using System.Collections;
	using System.ComponentModel;
	using System.Data;
	using System.Drawing;
	using System.Windows.Forms;
	using HSS.Data;
	#endregion

	#region ModuleBase
	/// <summary>
	/// Provides an empty Base Module that can be used to create other Modules
	/// </summary>
	[ToolboxItem(false)]
	public class ModuleBase : System.Windows.Forms.UserControl, ICanBeDirty
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		/// <summary>
		/// Represents the default text in a ComboBox
		/// </summary>
		public const string ComboBoxSelectText = "<Select Item>";
		/// <summary>
		/// Represents the default value in a ComboBox
		/// </summary>
		public const string ComboBoxSelectValue = "selectitem";
		/// <summary>
		/// Represents the Configuration is not available text for a ComboBox
		/// </summary>
		public const string PropertyNotAvailable = "N/A";


		/// <summary>
		/// Default Constructor
		/// </summary>
		public ModuleBase()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion

		#region MessageBox Helper
		/// <summary>
		/// Display a message box
		/// </summary>
		/// <param name="message">The message to display</param>
		public DialogResult MsgBox(string message)
		{
			return MsgBox(message, MessageBoxButtons.OK, MessageBoxIcon.Error);
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="message">The message to display</param>
		/// <param name="caption">The caption for the window</param>
		public DialogResult MsgBox(string message, string caption)
		{
			return MsgBox(message, caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
		}
		/// <summary>
		/// Display a message box
		/// </summary>
		/// <param name="message">The message to display</param>
		/// <param name="button">The button to display</param>
		/// <param name="icon">The icon to display</param>
		public DialogResult MsgBox(string message, MessageBoxButtons button, MessageBoxIcon icon)
		{
			return MsgBox(message, Application.ProductName, button, icon);
		}
		/// <summary>
		/// Display a message box
		/// </summary>
		/// <param name="message">The message to display</param>
		/// <param name="caption">The caption for the window</param>
		/// <param name="button">The button to display</param>
		/// <param name="icon">The icon to display</param>
		public DialogResult MsgBox(string message, string caption, MessageBoxButtons button, MessageBoxIcon icon)
		{
			return MessageBox.Show(message, caption, button, icon);
		}

		#endregion

		#region ComboBox Helper

		/// <summary>
		/// InitializeComboBox
		/// </summary>
		/// <param name="ctl">The ComboBox to initialize</param>
		/// <param name="dt">The DataTable to use as the datasource</param>
		/// <param name="DisplayColumn">The Column Name that should be displayed</param>
		/// <param name="ValueColumn">The Column Name that should be stored</param>
		/// <param name="SetDefault">Whether or not to set the list to a default list item</param>
		public static void InitializeComboBox(System.Windows.Forms.ComboBox ctl,
			System.Data.DataTable dt,
			string DisplayColumn,
			string ValueColumn,
			bool SetDefault)
		{

			if (null == ctl)
				return;

			ArrayList vm = new ArrayList();

			if (null != dt)
			{
				foreach (System.Data.DataRow r in dt.Rows)
					vm.Add(new ValueMember(r[DisplayColumn].ToString(), r[ValueColumn].ToString()));
			}

			if (vm.Count == 0)
			{
				ctl.ValueMember = null;
				ctl.DisplayMember = null;
				ctl.DataSource = null;
			}
			else
			{

				if (ctl.ValueMember != "Value")
					ctl.ValueMember = "Value";

				if (ctl.DisplayMember != "TextDisplay")
					ctl.DisplayMember = "TextDisplay";

				ctl.DataSource = vm.Clone();

				if (SetDefault)
					ctl.SelectedIndex = 0;
				else
					ctl.SelectedIndex = -1;
			}

			ctl.Refresh();
		}

		/// <summary>
		/// InitializeComboBox
		/// </summary>
		/// <param name="ctl">The ComboBox to initialize</param>
		/// <param name="dt">The DataTable to use as the datasource</param>
		/// <param name="DisplayColumn">The Column Name that should be displayed</param>
		/// <param name="ValueColumn">The Column Name that should be stored</param>
		public static void InitializeComboBox(System.Windows.Forms.ComboBox ctl,
			System.Data.DataTable dt,
			string DisplayColumn,
			string ValueColumn)
		{

			if (null == ctl)
				return;

			InitializeComboBox(ctl, dt, DisplayColumn, ValueColumn, true);

		}

		/// <summary>
		/// InitializeComboBox
		/// </summary>
		/// <param name="ctl">The ComboBox to initialize</param>
		/// <param name="dt">The DataTable to use as the datasource</param>
		/// <param name="PreserveSelectValue">Preserve the previous selected item</param>
		/// <param name="DisplayColumn">The Column Name that should be displayed</param>
		/// <param name="ValueColumn">The Column Name that should be stored</param>
		public static void InitializeComboBox(System.Windows.Forms.ComboBox ctl,
			System.Data.DataTable dt,
			bool PreserveSelectValue,
			string DisplayColumn,
			string ValueColumn)
		{

			if (null == ctl)
				return;

			ValueMember ExistingValue = null;
			if (PreserveSelectValue && null != ctl.SelectedValue)
				ExistingValue = new ValueMember(ctl.SelectedText, ctl.SelectedValue.ToString());

			ArrayList vm = new ArrayList();

			if (null != dt)
			{
				foreach (System.Data.DataRow r in dt.Rows)
					vm.Add(new ValueMember(r[DisplayColumn].ToString(), r[ValueColumn].ToString()));
			}

			if (vm.Count == 0)
			{
				ctl.ValueMember = null;
				ctl.DisplayMember = null;
				ctl.DataSource = null;
			}
			else
			{

				if (ctl.ValueMember != "Value")
					ctl.ValueMember = "Value";

				if (ctl.DisplayMember != "TextDisplay")
					ctl.DisplayMember = "TextDisplay";

				ctl.DataSource = vm.Clone();

				bool found = false;
				if (PreserveSelectValue && (null != ExistingValue))
				{
					foreach (object item in ctl.Items)
					{
						if (((ValueMember)item).Value == ExistingValue.Value)
						{
							ctl.SelectedValue = ExistingValue.Value;
							found = true;
							break;
						}
					}
					if (!found)
					{
						ctl.SelectedValue = ((ValueMember)ctl.Items[0]).Value;
						found = true;
					}
				}
				if (!found)
				{
					ctl.SelectedIndex = 0;
				}
			}

			ctl.Refresh();
		}

		/// <summary>
		/// InitializeComboBox
		/// </summary>
		/// <param name="ctl">The ComboBox to initialize</param>
		/// <param name="dr">The List to use as the datasource</param>
		public static void InitializeComboBox(System.Windows.Forms.ComboBox ctl,
			System.Collections.IDictionary dr)
		{

			if (null == ctl)
				return;

			ArrayList vm = new ArrayList();

			if (null != dr)
			{
				foreach (DictionaryEntry de in dr)
					vm.Add(new ValueMember(de.Value.ToString(), de.Key.ToString()));
			}

			if (vm.Count == 0)
			{
				ctl.ValueMember = null;
				ctl.DisplayMember = null;
				ctl.DataSource = null;
			}
			else
			{

				if (ctl.ValueMember != "Value")
					ctl.ValueMember = "Value";

				if (ctl.DisplayMember != "TextDisplay")
					ctl.DisplayMember = "TextDisplay";

				ctl.DataSource = vm.Clone();
			}

			ctl.Refresh();
		}

		/// <summary>
		/// InitializeComboBox
		/// </summary>
		/// <param name="ctl">The ComboBox to initialize</param>
		/// <param name="dr">The List to use as the datasource</param>
		/// <param name="ShowSelect">Show select</param>
		public static void InitializeComboBox(System.Windows.Forms.ComboBox ctl,
			System.Collections.IDictionary dr,
			bool ShowSelect)
		{

			if (null == ctl)
				return;

			ArrayList vm = new ArrayList();

			if (null != dr)
			{
				if (ShowSelect)
					vm.Add(new ValueMember(ComboBoxSelectText, ComboBoxSelectValue));

				foreach (DictionaryEntry de in dr)
					vm.Add(new ValueMember(de.Value.ToString(), de.Key.ToString()));
			}

			if (vm.Count == 0)
			{
				ctl.ValueMember = null;
				ctl.DisplayMember = null;
				ctl.DataSource = null;
			}
			else
			{

				if (ctl.ValueMember != "Value")
					ctl.ValueMember = "Value";

				if (ctl.DisplayMember != "TextDisplay")
					ctl.DisplayMember = "TextDisplay";

				ctl.DataSource = vm.Clone();
			}

			ctl.Refresh();
		}

		/// <summary>
		/// InitializeComboBox
		/// </summary>
		/// <param name="ctl"></param>
		/// <param name="pair">An array of Tuples containing the values for the combobox. The first item is the name, the second item is the value.</param>
		/// <param name="ShowSelect">Show select</param>
		public static void InitializeComboBox(System.Windows.Forms.ComboBox ctl,
			Tuple<string, string>[] pair,
			bool ShowSelect)
		{

			if (null == ctl)
				return;

			ArrayList vm = new ArrayList();

			if (null != pair)
			{
				if (ShowSelect)
					vm.Add(new ValueMember(ComboBoxSelectText, ComboBoxSelectValue));

				foreach (Tuple<string, string> de in pair)
					vm.Add(new ValueMember(de.Item1, de.Item2));
			}

			if (vm.Count == 0)
			{
				ctl.ValueMember = null;
				ctl.DisplayMember = null;
				ctl.DataSource = null;
			}
			else
			{

				if (ctl.ValueMember != "Value")
					ctl.ValueMember = "Value";

				if (ctl.DisplayMember != "TextDisplay")
					ctl.DisplayMember = "TextDisplay";

				ctl.DataSource = vm.Clone();
			}

			ctl.Refresh();
		}

		/// <summary>
		/// InitializeComboBox
		/// </summary>
		/// <param name="ctl">The ComboBox to initialize</param>
		/// <param name="dr">The DataRow array to use as the datasource</param>
		/// <param name="DisplayIndex">The index of the column to Display</param>
		/// <param name="ValueIndex">The index of the column to use for the Value</param>
		public static void InitializeComboBox(System.Windows.Forms.ComboBox ctl,
			System.Data.DataRow[] dr,
			int DisplayIndex,
			int ValueIndex)
		{

			if (null == ctl)
				return;

			ArrayList vm = new ArrayList();

			if (null != dr)
			{
				foreach (DataRow de in dr)
					vm.Add(new ValueMember(de.ItemArray[DisplayIndex].ToString(), de.ItemArray[ValueIndex].ToString()));
			}

			if (vm.Count == 0)
			{
				ctl.ValueMember = null;
				ctl.DisplayMember = null;
				ctl.DataSource = null;
			}
			{

				if (ctl.ValueMember != "Value")
					ctl.ValueMember = "Value";

				if (ctl.DisplayMember != "TextDisplay")
					ctl.DisplayMember = "TextDisplay";

				ctl.DataSource = vm.Clone();
			}

			ctl.Refresh();
		}


		/// <summary>
		/// InitializeComboBox
		/// </summary>
		/// <param name="ctl">The ComboBox to initialize</param>
		/// <param name="dr">The DataRowReader to use as the datasource</param>
		/// <param name="DisplayName">The index of the column to Display</param>
		/// <param name="ValueName">The index of the column to use for the Value</param>
		public static void InitializeComboBox(System.Windows.Forms.ComboBox ctl,
			DataRowReader dr,
			string DisplayName,
			string ValueName)
		{

			if (null == ctl)
				return;

			ArrayList vm = new ArrayList();

			if (null != dr)
			{
				while (dr.Read())
				{
					vm.Add(new ValueMember(dr[DisplayName].ToString(), dr[ValueName].ToString()));
				}
			}

			if (vm.Count == 0)
			{
				ctl.ValueMember = null;
				ctl.DisplayMember = null;
				ctl.DataSource = null;
			}
			{

				if (ctl.ValueMember != "Value")
					ctl.ValueMember = "Value";

				if (ctl.DisplayMember != "TextDisplay")
					ctl.DisplayMember = "TextDisplay";

				ctl.DataSource = vm.Clone();
			}

			ctl.Refresh();
		}

		/// <summary>
		/// InitializeComboBox
		/// </summary>
		/// <param name="ctl">The ComboBox to initialize</param>
		/// <param name="dr">The DataRowReader to use as the datasource</param>
		/// <param name="DisplayName">The index of the column to Display</param>
		/// <param name="ValueName">The index of the column to use for the Value</param>
		/// <param name="AdditionalValues">Additional values</param>
		/// <param name="AdditionalValueKeys">Additional value keys</param>
		public static void InitializeComboBox(System.Windows.Forms.ComboBox ctl,
			DataRowReader dr,
			string DisplayName,
			string ValueName,
			string[] AdditionalValues,
			string[] AdditionalValueKeys)
		{

			if (null == ctl)
				return;

			ArrayList vm = new ArrayList();

			if (null != dr)
			{
				if (null != AdditionalValues)
				{
					if (AdditionalValues.Length == AdditionalValueKeys.Length)
					{
						for (int item = 0; item < AdditionalValues.Length; item++)
						{
							vm.Add(new ValueMember(AdditionalValues[item], AdditionalValueKeys[item]));
						}
					}
				}

				while (dr.Read())
				{
					vm.Add(new ValueMember(dr[DisplayName].ToString(), dr[ValueName].ToString()));
				}
			}

			if (vm.Count == 0)
			{
				ctl.ValueMember = null;
				ctl.DisplayMember = null;
				ctl.DataSource = null;
			}
			{

				if (ctl.ValueMember != "Value")
					ctl.ValueMember = "Value";

				if (ctl.DisplayMember != "TextDisplay")
					ctl.DisplayMember = "TextDisplay";

				ctl.DataSource = vm.Clone();
			}

			ctl.Refresh();
		}

		/// <summary>
		/// InitializeComboBox
		/// </summary>
		/// <param name="ctl">The ComboBox to initialize</param>
		/// <param name="configData">The IDataContainer array to use as the datasource</param>
		/// <param name="ShowSelect">Add the Select Item as the first Item</param>
		/// <param name="PreserveSelectValue">Preserve the previous selected item</param>
		public static void InitializeComboBox(System.Windows.Forms.ComboBox ctl,
			IDataContainer[] configData,
			bool ShowSelect,
			bool PreserveSelectValue)
		{
			InitializeComboBox(ctl, configData, ShowSelect, PreserveSelectValue, null, null);
		}

		/// <summary>
		/// InitializeComboBox
		/// </summary>
		/// <param name="ctl">ComboBox</param>
		/// <param name="configData">ConfigurationDataSource</param>
		/// <param name="ShowSelect">Show select</param>
		/// <param name="PreserveSelectValue">Preserve existing value</param>
		/// <param name="AdditionalValues">Additional values</param>
		/// <param name="AdditionalValueKeys">Additional value keys</param>
		public static void InitializeComboBox(System.Windows.Forms.ComboBox ctl,
			IDataContainer[] configData,
			bool ShowSelect,
			bool PreserveSelectValue,
			string[] AdditionalValues,
			string[] AdditionalValueKeys)
		{

			if (null == ctl)
				return;

			ValueMember ExistingValue = null;
			if (PreserveSelectValue && null != ctl.SelectedValue)
				ExistingValue = new ValueMember(ctl.SelectedValue.ToString(), ctl.SelectedValue.ToString());

			ArrayList vm = new ArrayList();

			if (null != configData)
			{
				if (ShowSelect)
					vm.Add(new ValueMember(ComboBoxSelectText, ComboBoxSelectValue));

				if (null != AdditionalValues)
				{
					if (AdditionalValues.Length == AdditionalValueKeys.Length)
					{
						for (int item = 0; item < AdditionalValues.Length; item++)
						{
							vm.Add(new ValueMember(AdditionalValues[item], AdditionalValueKeys[item]));
						}
					}
				}

				foreach (IDataContainer data in configData)
					vm.Add(new ValueMember(data.Name, data.Data));
			}

			if (vm.Count == 0)
			{
				ctl.ValueMember = null;
				ctl.DisplayMember = null;
				ctl.DataSource = null;
			}
			else
			{

				if (ctl.ValueMember != "Value")
					ctl.ValueMember = "Value";

				if (ctl.DisplayMember != "TextDisplay")
					ctl.DisplayMember = "TextDisplay";

				ctl.DataSource = vm.Clone();

				bool found = false;
				if (PreserveSelectValue && (null != ExistingValue))
				{
					foreach (object item in ctl.Items)
					{
						if (((ValueMember)item).Value == ExistingValue.Value)
						{
							ctl.SelectedValue = ExistingValue.Value;
							found = true;
							break;
						}
					}
					if (!found)
					{
						ctl.SelectedValue = ((ValueMember)ctl.Items[0]).Value;
						found = true;
					}
				}
				if (!found)
				{
					ctl.SelectedIndex = 0;
				}
			}

			ctl.Refresh();
		}

		#endregion

		#region Child Controls Helper

		/// <summary>
		/// Properly invokes a delegate allowing for a WinForm thread
		/// </summary>
		/// <param name="del">The delegate to call</param>
		/// <param name="args">Any arguments to be passed</param>
		public static void InvokeDelegate(Delegate del, object[] args)
		{
			System.ComponentModel.ISynchronizeInvoke synchronizer;
			synchronizer = del.Target as System.ComponentModel.ISynchronizeInvoke;
			if (synchronizer != null) //A Windows Forms object
			{
				if (synchronizer.InvokeRequired == false)
				{
					del.DynamicInvoke(args);
					return;
				}
				try
				{
					synchronizer.Invoke(del, args);
				}
				catch
				{ }
			}
			else //Not a Windows Forms object
			{
				del.DynamicInvoke(args);
			}
		}

		/// <summary>
		/// The font to set the label of a de-selected Control
		/// </summary>
		/// <value>Font</value>
		[Category("Appearance")]
		[Description("The font to set the label of a de-selected Control")]
		public Font DeselectedFont
		{
			get
			{
				return _deselectedFont;
			}
			set
			{
				_deselectedFont = value;
			}
		} Font _deselectedFont = new Font("Arial Unicode MS", 8, FontStyle.Regular);

		/// <summary>
		/// The color to set the label of a de-selected Control
		/// </summary>
		/// <value>Color</value>
		[Category("Appearance")]
		[Description("The color to set the label of a de-selected Control")]
		public Color DeselectedForeColor
		{
			get
			{
				return _deselectedForeColor;
			}
			set
			{
				_deselectedForeColor = value;
			}
		} Color _deselectedForeColor = Color.Gray;

		/// <summary>
		/// The font to set the label of a selected Control
		/// </summary>
		/// <value>Font</value>
		[Category("Appearance")]
		[Description("The font to set the label of a selected Control")]
		public Font SelectedFont
		{
			get
			{
				return _selectedFont;
			}
			set
			{
				_selectedFont = value;
			}
		} Font _selectedFont = new Font("Verdana", 9, FontStyle.Regular);

		/// <summary>
		/// The color to set the label of a selected Control
		/// </summary>
		/// <value>Color</value>
		[Category("Appearance")]
		[Description("The color to set the label of a selected Control")]
		public Color SelectedForeColor
		{
			get
			{
				return _selectedForeColor;
			}
			set
			{
				_selectedForeColor = value;
			}
		} Color _selectedForeColor = Color.IndianRed;

		/// <summary>
		/// A List Array of Controls to skip when deactivating and setting label font
		/// </summary>
		[Browsable(false)]
		public System.Collections.Generic.List<Control> ControlsToSkip
		{
			get { return _controlsToSkip; }
			set { _controlsToSkip = value; }
		} System.Collections.Generic.List<Control> _controlsToSkip = new System.Collections.Generic.List<Control>();

		/// <summary>
		/// Reset the Children Controls of the Control Collection Parameter
		/// </summary>
		/// <param name="ParentCollection">The Control Collection to search.</param>
		public void ResetControls(Control.ControlCollection ParentCollection)
		{
			foreach (Control foundControl in ParentCollection)
			{
				if (foundControl.HasChildren)
					ResetControls(foundControl.Controls);
				else
				{
					if (foundControl.GetType() == typeof(TextBox))
						((TextBox)foundControl).Text = string.Empty;
					else if (foundControl.GetType() == typeof(RichTextBox))
						((RichTextBox)foundControl).Text = string.Empty;
					else if (foundControl.GetType() == typeof(DateTimePicker))
						((DateTimePicker)foundControl).Checked = false;
					else if (foundControl.GetType() == typeof(RadioButton))
						((RadioButton)foundControl).Checked = false;
					else if (foundControl.GetType() == typeof(ComboBox))
					{
						if (((ComboBox)foundControl).Items.Count > 0)
							((ComboBox)foundControl).SelectedIndex = 0;
						else
							((ComboBox)foundControl).SelectedIndex = -1;
					}
				}
			}
		}

		/// <summary>
		/// FindControl
		/// </summary>
		/// <param name="ParentCollection">ControlCollection to Search</param>
		/// <param name="ControlToFind">The Control to find</param>
		/// <returns>The Control found else (Control)null</returns>
		public Control FindControl(Control.ControlCollection ParentCollection, Control ControlToFind)
		{
			foreach (Control foundControl in ParentCollection)
			{
				if (foundControl.HasChildren)
				{
					Control tmpControl = FindControl(foundControl.Controls, ControlToFind);
					if (null != tmpControl && tmpControl.Name == ControlToFind.Name)
						return tmpControl;
				}
				else
				{
					if (foundControl.Name == ControlToFind.Name)
						return foundControl;
				}
			}
			return null;
		}

		/// <summary>
		/// DeactivateLabels
		/// </summary>
		public void DeactivateLabels()
		{
			DeactivateLabels(this.Controls);
		}
		/// <summary>
		/// DeactivateLabels
		/// </summary>
		/// <param name="ParentCollection">ControlCollection to Search</param>
		public void DeactivateLabels(Control.ControlCollection ParentCollection)
		{
			bool continueSearching = false;
			foreach (Control foundControl in ParentCollection)
			{
				continueSearching = false;

				foreach (object ctl in this._controlsToSkip)
				{
					if (((Control)ctl).Name == foundControl.Name)
					{
						continueSearching = true;
						break;
					}
				}
				if (!continueSearching)
				{
					if (foundControl.HasChildren)
					{
						DeactivateLabels(foundControl.Controls);
					}
					else
					{
						if (foundControl.GetType() == typeof(Label))
						{
							((Label)foundControl).Font = DeselectedFont;
							((Label)foundControl).ForeColor = DeselectedForeColor;
						}
						else if (foundControl.GetType() == typeof(RadioButton))
						{
							if (!((RadioButton)foundControl).Checked || !((RadioButton)foundControl).Enabled)
							{
								((RadioButton)foundControl).Font = DeselectedFont;
								((RadioButton)foundControl).ForeColor = DeselectedForeColor;
							}
							else
							{
								((RadioButton)foundControl).Font = SelectedFont;
								((RadioButton)foundControl).ForeColor = SelectedForeColor;
							}
						}
						else if (foundControl.GetType() == typeof(CheckBox))
						{
							if (!((CheckBox)foundControl).Checked || !((CheckBox)foundControl).Enabled)
							{
								((CheckBox)foundControl).Font = DeselectedFont;
								((CheckBox)foundControl).ForeColor = DeselectedForeColor;
							}
							else
							{
								((CheckBox)foundControl).Font = SelectedFont;
								((CheckBox)foundControl).ForeColor = SelectedForeColor;
							}
						}
					}
				}
			}
		}

		/// <summary>
		/// SetLabels
		/// </summary>
		/// <param name="SelectedControl">The Control to indicate as active</param>
		/// <remarks>This is for simple use only. Assume all child Controls are children of the Control.Controls. If
		/// you have Panels or other Container Controls that have children Controls, you may want to use the UpdateLabels
		/// as it provide more detail Control on how you activate and deacitvate Control. Also use the UpdateLabels method
		/// if you want to skip Controls. This method acts on all Controls.</remarks>
		public void SetLabels(Control SelectedControl)
		{
			UpdateLabels(this.Controls, SelectedControl);
		}
		/// <summary>
		/// UpdateLabel
		/// </summary>
		/// <param name="ParentControl">ControlCollection to Search</param>
		/// <param name="SelectedControl">The Control label to update</param>
		public void UpdateLabels(Control.ControlCollection ParentControl, Control SelectedControl)
		{
			DeactivateLabels(ParentControl);

			Control foundControl = FindControl(ParentControl, SelectedControl);

			if (null != foundControl)
			{
				if (foundControl.GetType() == typeof(Label))
				{
					Label c = (Label)SelectedControl;
					c.Font = SelectedFont;
					c.ForeColor = SelectedForeColor;
				}
				else if (foundControl.GetType() == typeof(RadioButton))
				{
					RadioButton c = (RadioButton)SelectedControl;
					c.Font = SelectedFont;
					c.ForeColor = SelectedForeColor;
				}
				else if (foundControl.GetType() == typeof(CheckBox))
				{
					CheckBox c = (CheckBox)SelectedControl;
					c.Font = SelectedFont;
					c.ForeColor = SelectedForeColor;
				}
				else if (foundControl.GetType() == typeof(DateTimePicker))
				{
					DateTimePicker c = (DateTimePicker)SelectedControl;
					c.Font = SelectedFont;
					c.ForeColor = SelectedForeColor;
				}
			}
		}

		#endregion

		#region ICanBeDirty Members
		/// <summary>
		/// Dirty Event
		/// </summary>
		public event EventHandler<DirtyEventArgs> DirtyChanged;
		/// <summary>
		/// InvokeDirty Event on Clients
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected virtual void InvokeDirty(object sender, DirtyEventArgs e)
		{
			if (null != this.DirtyChanged)
			{
				System.Delegate[] dlgs = this.DirtyChanged.GetInvocationList();
				if (null != dlgs)
				{
					foreach (Delegate d in dlgs)
					{
						if (null != d)
						{
							InvokeDelegate(d, new object[2] { sender, e });
						}
					}
				}
			}
		}
		/// <summary>
		/// Indicates if this form or control has been modified
		/// </summary>
		[Description("Indicates if this form or control has been modified.")]
		[Browsable(false)]
		[DefaultValue(false)]
		public bool IsDirty
		{
			get { return _dirty; }
			set { _dirty = value; }
		} bool _dirty;

		#endregion
	}
	#endregion
}