// HSS.Logging.Management.ConfigurationEditor.cs
// ----------------------------------------------------------------------------
// Licensed under the MIT license
// http://www.opensource.org/licenses/mit-license.html
// ----------------------------------------------------------------------------
// HighSpeed-Solutions, LLC
// Copyright (c) 2001-2010
// ----------------------------------------------------------------------------
// File:       ConfigurationEditor.cs
// Author:     HSS\gbanta
// Created:    08/12/2010
// Modified:   12/04/2010
// ----------------------------------------------------------------------------
namespace HSS.Logging.Management
{
	#region Using Directives
	using System;
	using System.Text;
	using System.Windows.Forms;
	using HSS.Forms;
	using System.Security.Permissions;
	#endregion

	#region ConfigurationEditor
	/// <summary>
	/// Provides a user interface allowing an administrator to configure the Logging System.
	/// </summary>
	public sealed partial class ConfigurationEditor : HSS.Forms.DetailModuleContainer
	{
		/// <summary>
		/// ctor
		/// </summary>
		internal ConfigurationEditor()
		{
			InitializeComponent();
		}

		/// <summary>
		/// Runs the Configuration Editor as a stand alone application
		/// </summary>
		[STAThread()]
#if !DEBUG
		[PrincipalPermissionAttribute(SecurityAction.Demand, Role = "Administrators")]
#endif
		public static void Run()
		{
			using (var ctx = new AppContext<ConfigurationEditor>())
			{
				ctx.RestoreLocation = false;
				ctx.RestoreSize = false;
				ctx.RestoreState = false;
				ctx.Run(null, InitApp);
			}
		}
		private static ConfigurationEditor InitApp(string[] args, AppContext<ConfigurationEditor> ctx)
		{
			ctx.ShowError += new EventHandler<SendExceptionEventArgs>(App_ShowError);
			ctx.SendException += new EventHandler<SendExceptionEventArgs>(App_SendException);

			var mainForm = new ConfigurationEditor();
			if (ConfigurationEditor.LoadInstance(mainForm, typeof(ConfigurationEditorControl), "EventLog Configuration", null, ModuleActions.Edit, false, null) != DialogResult.Cancel)
				return mainForm;
			return null;
		}
		private static void App_ShowError(object sender, SendExceptionEventArgs e)
		{
			MessageBox.Show(e.UnhandledException.ToString(), "Exception Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
		}
		private static void App_SendException(object sender, SendExceptionEventArgs e)
		{
			if (e.SendException)
				SendException(e.UnhandledException);

			if (e.RestartApp)
				Application.Restart();
		}
		private static void SendException(Exception ex)
		{
			SendException(ex.ToString());
		}
		private static void SendException(string message)
		{
			System.Threading.ThreadPool.QueueUserWorkItem(SendException, message);
		}
		private static void SendException(object objMessage)
		{
			try
			{
				var builder = new StringBuilder(objMessage.ToString());
				builder.AppendLine();
				builder.AppendLine();
				AppContext<ConfigurationEditor>.AppendAppContextInfo(builder);
				builder.AppendLine();
				builder.AppendLine();
				AppContext<ConfigurationEditor>.AppendEnvironmentInfo(builder);
				var subject = Application.ProductName + ": " + HSS.Logging.Management.Resources.supportSubject;
				var msg = new System.Net.Mail.MailMessage(HSS.Logging.Management.Resources.suppportEmail, HSS.Logging.Management.Resources.suppportEmail, subject, builder.ToString());
				HSS.Net.Mail.SmtpClient.SendEmail(msg);
			}
			catch { }
		}
	}
	#endregion
}