// HSS.Net.Dns.Structs.cs
// ----------------------------------------------------------------------------
// Licensed under the MIT license
// http://www.opensource.org/licenses/mit-license.html
// ----------------------------------------------------------------------------
// HighSpeed-Solutions, LLC
// Copyright (c) 2001-2010
// ----------------------------------------------------------------------------
// File:       Structs.cs
// Author:     HSS\gbanta
// Created:    08/12/2010
// Modified:   12/04/2010
// ----------------------------------------------------------------------------
namespace HSS.Net.Dns
{
	#region Types
	/*
	 * 3.2.2. TYPE values
	 *
	 * TYPE fields are used in resource records.
	 * Note that these types are a subset of QTYPEs.
	 *
	 *		TYPE		value			meaning
	 */
	/// <summary>
	/// TYPE fields are used in resource records
	/// </summary>
	[System.CLSCompliant(false)]
	public enum Type : ushort
	{
		/// <summary>
		/// a IPV4 host address
		/// </summary>
		A = 1,				// a IPV4 host address
		/// <summary>
		/// an authoritative name server
		/// </summary>
		NS = 2,				// an authoritative name server
		/// <summary>
		/// a mail destination (Obsolete - use MX)
		/// </summary>
		MD = 3,				// a mail destination (Obsolete - use MX)
		/// <summary>
		/// a mail forwarder (Obsolete - use MX)
		/// </summary>
		MF = 4,				// a mail forwarder (Obsolete - use MX)
		/// <summary>
		/// the canonical name for an alias
		/// </summary>
		CNAME = 5,			// the canonical name for an alias
		/// <summary>
		/// marks the start of a zone of authority
		/// </summary>
		SOA = 6,			// marks the start of a zone of authority
		/// <summary>
		/// a mailbox domain name (EXPERIMENTAL)
		/// </summary>
		MB = 7,				// a mailbox domain name (EXPERIMENTAL)
		/// <summary>
		/// a mail group member (EXPERIMENTAL)
		/// </summary>
		MG = 8,				// a mail group member (EXPERIMENTAL)
		/// <summary>
		/// a mail rename domain name (EXPERIMENTAL)
		/// </summary>
		MR = 9,				// a mail rename domain name (EXPERIMENTAL)
		/// <summary>
		/// a null RR (EXPERIMENTAL)
		/// </summary>
		NULL = 10,			// a null RR (EXPERIMENTAL)
		/// <summary>
		/// a well known service description
		/// </summary>
		WKS = 11,			// a well known service description
		/// <summary>
		/// a domain name pointer
		/// </summary>
		PTR = 12,			// a domain name pointer
		/// <summary>
		/// host information
		/// </summary>
		HINFO = 13,			// host information
		/// <summary>
		/// mailbox or mail list information
		/// </summary>
		MINFO = 14,			// mailbox or mail list information
		/// <summary>
		/// mail exchange
		/// </summary>
		MX = 15,			// mail exchange
		/// <summary>
		/// text strings
		/// </summary>
		TXT = 16,			// text strings
		/// <summary>
		/// The Responsible Person rfc1183
		/// </summary>
		RP = 17,			// The Responsible Person rfc1183
		/// <summary>
		/// AFS Data Base location
		/// </summary>
		AFSDB = 18,			// AFS Data Base location
		/// <summary>
		/// X.25 address rfc1183
		/// </summary>
		X25 = 19,			// X.25 address rfc1183
		/// <summary>
		/// ISDN address rfc1183 
		/// </summary>
		ISDN = 20,			// ISDN address rfc1183 
		/// <summary>
		/// The Route Through rfc1183
		/// </summary>
		RT = 21,			// The Route Through rfc1183
		/// <summary>
		/// Network service access point address rfc1706
		/// </summary>
		NSAP = 22,			// Network service access point address rfc1706
		/// <summary>
		/// Obsolete, rfc1348
		/// </summary>
		NSAPPTR = 23,		// Obsolete, rfc1348
		/// <summary>
		/// Cryptographic public key signature rfc2931 / rfc2535
		/// </summary>
		SIG = 24,			// Cryptographic public key signature rfc2931 / rfc2535
		/// <summary>
		/// Public key as used in DNSSEC rfc2535
		/// </summary>
		KEY = 25,			// Public key as used in DNSSEC rfc2535
		/// <summary>
		/// Pointer to X.400/RFC822 mail mapping information rfc2163
		/// </summary>
		PX = 26,			// Pointer to X.400/RFC822 mail mapping information rfc2163
		/// <summary>
		/// Geographical position rfc1712 (obsolete)
		/// </summary>
		GPOS = 27,			// Geographical position rfc1712 (obsolete)
		/// <summary>
		/// a IPV6 host address, rfc3596
		/// </summary>
		AAAA = 28,			// a IPV6 host address, rfc3596
		/// <summary>
		/// Location information rfc1876
		/// </summary>
		LOC = 29,			// Location information rfc1876
		/// <summary>
		/// Next Domain, Obsolete rfc2065 / rfc2535
		/// </summary>
		NXT = 30,			// Next Domain, Obsolete rfc2065 / rfc2535
		/// <summary>
		/// *** Endpoint Identifier (Patton)
		/// </summary>
		EID = 31,			// *** Endpoint Identifier (Patton)
		/// <summary>
		/// *** Nimrod Locator (Patton)
		/// </summary>
		NIMLOC = 32,		// *** Nimrod Locator (Patton)
		/// <summary>
		/// Location of services rfc2782
		/// </summary>
		SRV = 33,			// Location of services rfc2782
		/// <summary>
		/// *** ATM Address (Dobrowski)
		/// </summary>
		ATMA = 34,			// *** ATM Address (Dobrowski)
		/// <summary>
		/// The Naming Authority Pointer rfc3403
		/// </summary>
		NAPTR = 35,			// The Naming Authority Pointer rfc3403
		/// <summary>
		/// Key Exchange Delegation Record rfc2230
		/// </summary>
		KX = 36,			// Key Exchange Delegation Record rfc2230
		/// <summary>
		/// *** CERT RFC2538
		/// </summary>
		CERT = 37,			// *** CERT RFC2538
		/// <summary>
		/// IPv6 address rfc3363 (rfc2874 rfc3226)
		/// </summary>
		A6 = 38,			// IPv6 address rfc3363 (rfc2874 rfc3226)
		/// <summary>
		/// A way to provide aliases for a whole domain, not just a single domain name as with CNAME. rfc2672
		/// </summary>
		DNAME = 39,			// A way to provide aliases for a whole domain, not just a single domain name as with CNAME. rfc2672
		/// <summary>
		/// *** SINK Eastlake
		/// </summary>
		SINK = 40,			// *** SINK Eastlake
		/// <summary>
		/// *** OPT RFC2671
		/// </summary>
		OPT = 41,			// *** OPT RFC2671
		/// <summary>
		/// *** APL [RFC3123]
		/// </summary>
		APL = 42,			// *** APL [RFC3123]
		/// <summary>
		/// Delegation Signer rfc3658
		/// </summary>
		DS = 43,			// Delegation Signer rfc3658
		/// <summary>
		/// SSH Key Fingerprint rfc4255
		/// </summary>
		SSHFP = 44,			// SSH Key Fingerprint rfc4255
		/// <summary>
		/// IPSECKEY rfc4025
		/// </summary>
		IPSECKEY = 45,		// IPSECKEY rfc4025
		/// <summary>
		/// RRSIG rfc3755
		/// </summary>
		RRSIG = 46,			// RRSIG rfc3755
		/// <summary>
		/// NSEC rfc3755
		/// </summary>
		NSEC = 47,			// NSEC rfc3755
		/// <summary>
		/// DNSKEY 3755
		/// </summary>
		DNSKEY = 48,		// DNSKEY 3755
		/// <summary>
		/// DHCID rfc4701
		/// </summary>
		DHCID = 49,			// DHCID rfc4701
		/// <summary>
		/// NSEC3 rfc5155
		/// </summary>
		NSEC3 = 50,			// NSEC3 rfc5155
		/// <summary>
		/// NSEC3PARAM rfc5155
		/// </summary>
		NSEC3PARAM = 51,	// NSEC3PARAM rfc5155
		/// <summary>
		/// Host Identity Protocol  [RFC-ietf-hip-dns-09.txt]
		/// </summary>
		HIP = 55,			// Host Identity Protocol  [RFC-ietf-hip-dns-09.txt]
		/// <summary>
		/// SPF rfc4408
		/// </summary>
		SPF = 99,			// SPF rfc4408
		/// <summary>
		/// *** IANA-Reserved
		/// </summary>
		UINFO = 100,		// *** IANA-Reserved
		/// <summary>
		/// *** IANA-Reserved
		/// </summary>
		UID = 101,			// *** IANA-Reserved
		/// <summary>
		/// *** IANA-Reserved
		/// </summary>
		GID = 102,			// *** IANA-Reserved
		/// <summary>
		/// *** IANA-Reserved
		/// </summary>
		UNSPEC = 103,		// *** IANA-Reserved
		/// <summary>
		/// Transaction key rfc2930
		/// </summary>
		TKEY = 249,			// Transaction key rfc2930
		/// <summary>
		/// Transaction signature rfc2845
		/// </summary>
		TSIG = 250,			// Transaction signature rfc2845
		/// <summary>
		/// DNSSEC Trust Authorities          [Weiler]  13 December 2005
		/// </summary>
		TA = 32768,			// DNSSEC Trust Authorities          [Weiler]  13 December 2005
		/// <summary>
		/// DNSSEC Lookaside Validation       [RFC4431]
		/// </summary>
		DLV = 32769			// DNSSEC Lookaside Validation       [RFC4431]
	}
	#endregion

	#region QTypes
	/*
	 * 3.2.3. QTYPE values
	 *
	 * QTYPE fields appear in the question part of a query.  QTYPES are a
	 * superset of TYPEs, hence all TYPEs are valid QTYPEs.  In addition, the
	 * following QTYPEs are defined:
	 *
	 *		QTYPE		value			meaning
	 */
	/// <summary>
	/// QTYPE fields appear in the question part of a query.  QTYPES are a
	/// superset of TYPEs, hence all TYPEs are valid QTYPEs.
	/// </summary>
	[System.CLSCompliant(false)]
	public enum QType : ushort
	{
		/// <summary>
		/// a IPV4 host address
		/// </summary>
		A = Type.A,			// a IPV4 host address
		/// <summary>
		/// an authoritative name server
		/// </summary>
		NS = Type.NS,		// an authoritative name server
		/// <summary>
		/// a mail destination (Obsolete - use MX)
		/// </summary>
		MD = Type.MD,		// a mail destination (Obsolete - use MX)
		/// <summary>
		/// a mail forwarder (Obsolete - use MX)
		/// </summary>
		MF = Type.MF,		// a mail forwarder (Obsolete - use MX)
		/// <summary>
		/// the canonical name for an alias
		/// </summary>
		CNAME = Type.CNAME,	// the canonical name for an alias
		/// <summary>
		/// marks the start of a zone of authority
		/// </summary>
		SOA = Type.SOA,		// marks the start of a zone of authority
		/// <summary>
		/// a mailbox domain name (EXPERIMENTAL)
		/// </summary>
		MB = Type.MB,		// a mailbox domain name (EXPERIMENTAL)
		/// <summary>
		/// a mail group member (EXPERIMENTAL)
		/// </summary>
		MG = Type.MG,		// a mail group member (EXPERIMENTAL)
		/// <summary>
		/// a mail rename domain name (EXPERIMENTAL)
		/// </summary>
		MR = Type.MR,		// a mail rename domain name (EXPERIMENTAL)
		/// <summary>
		/// a null RR (EXPERIMENTAL)
		/// </summary>
		NULL = Type.NULL,	// a null RR (EXPERIMENTAL)
		/// <summary>
		/// a well known service description
		/// </summary>
		WKS = Type.WKS,		// a well known service description
		/// <summary>
		/// a domain name pointer
		/// </summary>
		PTR = Type.PTR,		// a domain name pointer
		/// <summary>
		/// host information
		/// </summary>
		HINFO = Type.HINFO,	// host information
		/// <summary>
		/// mailbox or mail list information
		/// </summary>
		MINFO = Type.MINFO,	// mailbox or mail list information
		/// <summary>
		/// mail exchange
		/// </summary>
		MX = Type.MX,		// mail exchange
		/// <summary>
		/// text strings
		/// </summary>
		TXT = Type.TXT,		// text strings
		/// <summary>
		/// The Responsible Person rfc1183
		/// </summary>
		RP = Type.RP,		// The Responsible Person rfc1183
		/// <summary>
		/// AFS Data Base location
		/// </summary>
		AFSDB = Type.AFSDB,	// AFS Data Base location
		/// <summary>
		/// X.25 address rfc1183
		/// </summary>
		X25 = Type.X25,		// X.25 address rfc1183
		/// <summary>
		/// ISDN address rfc1183
		/// </summary>
		ISDN = Type.ISDN,	// ISDN address rfc1183
		/// <summary>
		/// The Route Through rfc1183
		/// </summary>
		RT = Type.RT,		// The Route Through rfc1183
		/// <summary>
		/// Network service access point address rfc1706
		/// </summary>
		NSAP = Type.NSAP,	// Network service access point address rfc1706
		/// <summary>
		/// Obsolete, rfc1348
		/// </summary>
		NSAP_PTR = Type.NSAPPTR, // Obsolete, rfc1348
		/// <summary>
		/// Cryptographic public key signature rfc2931 / rfc2535
		/// </summary>
		SIG = Type.SIG,		// Cryptographic public key signature rfc2931 / rfc2535
		/// <summary>
		/// Public key as used in DNSSEC rfc2535
		/// </summary>
		KEY = Type.KEY,		// Public key as used in DNSSEC rfc2535
		/// <summary>
		/// Pointer to X.400/RFC822 mail mapping information rfc2163
		/// </summary>
		PX = Type.PX,		// Pointer to X.400/RFC822 mail mapping information rfc2163
		/// <summary>
		/// Geographical position rfc1712 (obsolete)
		/// </summary>
		GPOS = Type.GPOS,	// Geographical position rfc1712 (obsolete)
		/// <summary>
		/// a IPV6 host address
		/// </summary>
		AAAA = Type.AAAA,	// a IPV6 host address
		/// <summary>
		/// Location information rfc1876
		/// </summary>
		LOC = Type.LOC,		// Location information rfc1876
		/// <summary>
		/// Obsolete rfc2065 / rfc2535
		/// </summary>
		NXT = Type.NXT,		// Obsolete rfc2065 / rfc2535
		/// <summary>
		/// *** Endpoint Identifier (Patton)
		/// </summary>
		EID = Type.EID,		// *** Endpoint Identifier (Patton)
		/// <summary>
		/// *** Nimrod Locator (Patton)
		/// </summary>
		NIMLOC = Type.NIMLOC,// *** Nimrod Locator (Patton)
		/// <summary>
		/// Location of services rfc2782
		/// </summary>
		SRV = Type.SRV,		// Location of services rfc2782
		/// <summary>
		/// *** ATM Address (Dobrowski)
		/// </summary>
		ATMA = Type.ATMA,	// *** ATM Address (Dobrowski)
		/// <summary>
		/// The Naming Authority Pointer rfc3403
		/// </summary>
		NAPTR = Type.NAPTR,	// The Naming Authority Pointer rfc3403
		/// <summary>
		/// Key Exchange Delegation Record rfc2230
		/// </summary>
		KX = Type.KX,		// Key Exchange Delegation Record rfc2230
		/// <summary>
		/// *** CERT RFC2538
		/// </summary>
		CERT = Type.CERT,	// *** CERT RFC2538
		/// <summary>
		/// IPv6 address rfc3363
		/// </summary>
		A6 = Type.A6,		// IPv6 address rfc3363
		/// <summary>
		/// A way to provide aliases for a whole domain, not just a single domain name as with CNAME. rfc2672
		/// </summary>
		DNAME = Type.DNAME,	// A way to provide aliases for a whole domain, not just a single domain name as with CNAME. rfc2672
		/// <summary>
		/// *** SINK Eastlake
		/// </summary>
		SINK = Type.SINK,	// *** SINK Eastlake
		/// <summary>
		/// *** OPT RFC2671
		/// </summary>
		OPT = Type.OPT,		// *** OPT RFC2671
		/// <summary>
		/// *** APL [RFC3123]
		/// </summary>
		APL = Type.APL,		// *** APL [RFC3123]
		/// <summary>
		/// Delegation Signer rfc3658
		/// </summary>
		DS = Type.DS,		// Delegation Signer rfc3658
		/// <summary>
		/// *** SSH Key Fingerprint RFC-ietf-secsh-dns
		/// </summary>
		SSHFP = Type.SSHFP,	// *** SSH Key Fingerprint RFC-ietf-secsh-dns
		/// <summary>
		/// rfc4025
		/// </summary>
		IPSECKEY = Type.IPSECKEY, // rfc4025
		/// <summary>
		/// *** RRSIG RFC-ietf-dnsext-dnssec-2535
		/// </summary>
		RRSIG = Type.RRSIG,	// *** RRSIG RFC-ietf-dnsext-dnssec-2535
		/// <summary>
		/// *** NSEC RFC-ietf-dnsext-dnssec-2535
		/// </summary>
		NSEC = Type.NSEC,	// *** NSEC RFC-ietf-dnsext-dnssec-2535
		/// <summary>
		/// *** DNSKEY RFC-ietf-dnsext-dnssec-2535
		/// </summary>
		DNSKEY = Type.DNSKEY,// *** DNSKEY RFC-ietf-dnsext-dnssec-2535
		/// <summary>
		/// rfc4701
		/// </summary>
		DHCID = Type.DHCID,	// rfc4701
		/// <summary>
		/// RFC5155
		/// </summary>
		NSEC3 = Type.NSEC3,	// RFC5155
		/// <summary>
		/// RFC5155
		/// </summary>
		NSEC3PARAM = Type.NSEC3PARAM, // RFC5155
		/// <summary>
		/// RFC-ietf-hip-dns-09.txt
		/// </summary>
		HIP = Type.HIP,		// RFC-ietf-hip-dns-09.txt
		/// <summary>
		/// RFC4408
		/// </summary>
		SPF = Type.SPF,		// RFC4408
		/// <summary>
		/// *** IANA-Reserved
		/// </summary>
		UINFO = Type.UINFO,	// *** IANA-Reserved
		/// <summary>
		/// *** IANA-Reserved
		/// </summary>
		UID = Type.UID,		// *** IANA-Reserved
		/// <summary>
		/// *** IANA-Reserved
		/// </summary>
		GID = Type.GID,		// *** IANA-Reserved
		/// <summary>
		/// *** IANA-Reserved
		/// </summary>
		UNSPEC = Type.UNSPEC,// *** IANA-Reserved
		/// <summary>
		/// Transaction key rfc2930
		/// </summary>
		TKEY = Type.TKEY,	// Transaction key rfc2930
		/// <summary>
		/// Transaction signature rfc2845
		/// </summary>
		TSIG = Type.TSIG,	// Transaction signature rfc2845
		/// <summary>
		/// [RFC1995]
		/// </summary>
		IXFR = 251,			// incremental transfer                  [RFC1995]
		/// <summary>
		/// an entire zone            [RFC1035]
		/// </summary>
		AXFR = 252,			// transfer of an entire zone            [RFC1035]
		/// <summary>
		/// related RRs (MB, MG or MR)    [RFC1035]
		/// </summary>
		MAILB = 253,		// mailbox-related RRs (MB, MG or MR)    [RFC1035]
		/// <summary>
		/// RRs (Obsolete - see MX)    [RFC1035]
		/// </summary>
		MAILA = 254,		// mail agent RRs (Obsolete - see MX)    [RFC1035]
		/// <summary>
		/// for all records             [RFC1035]
		/// </summary>
		ANY = 255,			// A request for all records             [RFC1035]
		/// <summary>
		/// DNSSEC Trust Authorities    [Weiler]  13 December 2005
		/// </summary>
		TA = Type.TA,		// DNSSEC Trust Authorities    [Weiler]  13 December 2005
		/// <summary>
		/// DNSSEC Lookaside Validation [RFC4431]
		/// </summary>
		DLV = Type.DLV		// DNSSEC Lookaside Validation [RFC4431]
	}
	#endregion

	#region Class
	/*
	 * 3.2.4. CLASS values
	 *
	 * CLASS fields appear in resource records.  The following CLASS mnemonics
	 * and values are defined:
	 *
	 *		CLASS		value			meaning
	 */
	/// <summary>
	/// CLASS fields appear in resource records
	/// </summary>
	[System.CLSCompliant(false)]
	public enum Class : ushort
	{
		/// <summary>
		/// the Internet
		/// </summary>
		IN = 1,				// the Internet
		/// <summary>
		/// the CSNET class (Obsolete - used only for examples in some obsolete RFCs)
		/// </summary>
		CS = 2,				// the CSNET class (Obsolete - used only for examples in some obsolete RFCs)
		/// <summary>
		/// the CHAOS class
		/// </summary>
		CH = 3,				// the CHAOS class
		/// <summary>
		/// Hesiod [Dyer 87]
		/// </summary>
		HS = 4				// Hesiod [Dyer 87]
	}
	#endregion

	#region QClass
	/*
	 * 3.2.5. QCLASS values
	 *
	 * QCLASS fields appear in the question section of a query.  QCLASS values
	 * are a superset of CLASS values; every CLASS is a valid QCLASS.  In
	 * addition to CLASS values, the following QCLASSes are defined:
	 *
	 *		QCLASS		value			meaning
	 */
	/// <summary>
	/// QCLASS fields appear in the question section of a query. QCLASS values
	/// are a superset of CLASS values; every CLASS is a valid QCLASS.
	/// </summary>
	[System.CLSCompliant(false)]
	public enum QClass : ushort
	{
		/// <summary>
		/// the Internet
		/// </summary>
		IN = Class.IN,		// the Internet
		/// <summary>
		/// the CSNET class (Obsolete - used only for examples in some obsolete RFCs)
		/// </summary>
		CS = Class.CS,		// the CSNET class (Obsolete - used only for examples in some obsolete RFCs)
		/// <summary>
		/// the CHAOS class
		/// </summary>
		CH = Class.CH,		// the CHAOS class
		/// <summary>
		/// Hesiod [Dyer 87]
		/// </summary>
		HS = Class.HS,		// Hesiod [Dyer 87]
		/// <summary>
		/// any class
		/// </summary>
		ANY = 255			// any class
	}
	#endregion

	#region Response Codes
	/*
		RCODE           
				Response code - this 4 bit field is set as part of
                responses.  The values have the following
                interpretation:
	 */
	/// <summary>
	/// Response code - this 4 bit field is set as part of responses
	/// </summary>
	[System.CLSCompliant(false)]
	public enum RCode
	{
		/// <summary>
		/// No Error                           [RFC1035]
		/// </summary>
		NoError = 0,		// No Error                           [RFC1035]
		/// <summary>
		/// Format Error                       [RFC1035]
		/// </summary>
		FormErr = 1,		// Format Error                       [RFC1035]
		/// <summary>
		/// Server Failure                     [RFC1035]
		/// </summary>
		ServFail = 2,		// Server Failure                     [RFC1035]
		/// <summary>
		/// Non-Existent Domain                [RFC1035]
		/// </summary>
		NXDomain = 3,		// Non-Existent Domain                [RFC1035]
		/// <summary>
		/// Not Implemented                    [RFC1035]
		/// </summary>
		NotImp = 4,			// Not Implemented                    [RFC1035]
		/// <summary>
		/// Query Refused                      [RFC1035]
		/// </summary>
		Refused = 5,		// Query Refused                      [RFC1035]
		/// <summary>
		/// Name Exists when it should not     [RFC2136]
		/// </summary>
		YXDomain = 6,		// Name Exists when it should not     [RFC2136]
		/// <summary>
		/// RR Set Exists when it should not   [RFC2136]
		/// </summary>
		YXRRSet = 7,		// RR Set Exists when it should not   [RFC2136]
		/// <summary>
		/// RR Set that should exist does not  [RFC2136]
		/// </summary>
		NXRRSet = 8,		// RR Set that should exist does not  [RFC2136]
		/// <summary>
		/// Server Not Authoritative for zone  [RFC2136]
		/// </summary>
		NotAuth = 9,		// Server Not Authoritative for zone  [RFC2136]
		/// <summary>
		/// Name not contained in zone         [RFC2136]
		/// </summary>
		NotZone = 10,		// Name not contained in zone         [RFC2136]
		/// <summary>
		/// Reserved
		/// </summary>
		RESERVED11 = 11,	// Reserved
		/// <summary>
		/// Reserved
		/// </summary>
		RESERVED12 = 12,	// Reserved
		/// <summary>
		/// Reserved
		/// </summary>
		RESERVED13 = 13,	// Reserved
		/// <summary>
		/// Reserved
		/// </summary>
		RESERVED14 = 14,	// Reserved
		/// <summary>
		/// Reserved
		/// </summary>
		RESERVED15 = 15,	// Reserved
		/// <summary>
		/// Bad OPT Version                    [RFC2671]
		/// </summary>
		BADVERSSIG = 16,	// Bad OPT Version                    [RFC2671]
		/// <summary>
		/// Key not recognized                 [RFC2845]
		/// </summary>
		BADKEY = 17,		// Key not recognized                 [RFC2845]
		/// <summary>
		/// Signature out of time window       [RFC2845]
		/// </summary>
		BADTIME = 18,		// Signature out of time window       [RFC2845]
		/// <summary>
		/// Bad TKEY Mode                      [RFC2930]
		/// </summary>
		BADMODE = 19,		// Bad TKEY Mode                      [RFC2930]
		/// <summary>
		/// Duplicate key name                 [RFC2930]
		/// </summary>
		BADNAME = 20,		// Duplicate key name                 [RFC2930]
		/// <summary>
		/// Algorithm not supported            [RFC2930]
		/// </summary>
		BADALG = 21,		// Algorithm not supported            [RFC2930]
		/// <summary>
		/// Truncation                     [RFC4635]
		/// </summary>
		BADTRUNC = 22		// Bad Truncation                     [RFC4635]

		/*
			23-3840              available for assignment
				0x0016-0x0F00
			3841-4095            Private Use
				0x0F01-0x0FFF
			4096-65535           available for assignment
				0x1000-0xFFFF
		*/

	}
	#endregion

	#region OPCode
	/* OPCODE
				A four bit field that specifies kind of query in this
                message.  This value is set by the originator of a query
                and copied into the response.  The values are:

                0               a standard query (QUERY)

                1               an inverse query (IQUERY)

                2               a server status request (STATUS)

                3-15            reserved for future use
	 */
	/// <summary>
	/// A four bit field that specifies kind of query in this 
	/// message. This value is set by the originator of a query 
	/// and copied into the response
	/// </summary>
	[System.CLSCompliant(false)]
	public enum OPCode
	{
		/// <summary>
		/// a standard query (QUERY)
		/// </summary>
		Query = 0,				// a standard query (QUERY)

		/// <summary>
		/// OpCode Retired (previously IQUERY - No further [RFC3425] assignment of this code available)
		/// </summary>
		IQUERY = 1,				// OpCode Retired (previously IQUERY - No further [RFC3425] assignment of this code available)

		/// <summary>
		/// a server status request (STATUS) RFC1035
		/// </summary>
		Status = 2,				// a server status request (STATUS) RFC1035

		/// <summary>
		/// IANA
		/// </summary>
		RESERVED3 = 3,			// IANA

		/// <summary>
		/// RFC1996
		/// </summary>
		Notify = 4,				// RFC1996

		/// <summary>
		/// RFC2136
		/// </summary>
		Update = 5,				// RFC2136

		/// <summary>
		/// RESERVED6
		/// </summary>
		RESERVED6 = 6,

		/// <summary>
		/// RESERVED7
		/// </summary>
		RESERVED7 = 7,

		/// <summary>
		/// RESERVED8
		/// </summary>
		RESERVED8 = 8,

		/// <summary>
		/// RESERVED9
		/// </summary>
		RESERVED9 = 9,

		/// <summary>
		/// RESERVED10
		/// </summary>
		RESERVED10 = 10,

		/// <summary>
		/// RESERVED11
		/// </summary>
		RESERVED11 = 11,

		/// <summary>
		/// RESERVED12
		/// </summary>
		RESERVED12 = 12,

		/// <summary>
		/// RESERVED13
		/// </summary>
		RESERVED13 = 13,

		/// <summary>
		/// RESERVED14
		/// </summary>
		RESERVED14 = 14,

		/// <summary>
		/// RESERVED15
		/// </summary>
		RESERVED15 = 15,
	}
	#endregion

	#region TransportType
	/// <summary>
	/// The Type of transport to use for Queries
	/// </summary>
	[System.CLSCompliant(false)]
	public enum TransportType
	{
		/// <summary>
		/// UDP Socket
		/// </summary>
		Udp,
		/// <summary>
		/// TCP Socket
		/// </summary>
		Tcp
	}
	#endregion
}