﻿// TestSharedData.UserPrincipal.cs
// ----------------------------------------------------------------------------
// Licensed under the MIT license
// http://www.opensource.org/licenses/mit-license.html
// ----------------------------------------------------------------------------
// HighSpeed-Solutions, LLC
// Copyright (c) 2001-2010
// ----------------------------------------------------------------------------
// File:       UserPrincipal.cs
// Author:     HSS\gbanta
// Created:    08/12/2010
// Modified:   12/04/2010
// ----------------------------------------------------------------------------
namespace TestSharedData
{
	#region Using Directives
	using System;
	using System.Security.Permissions;
	using System.Security.Principal;
	using System.Web;
	#endregion

	#region UserPrincipal
	/// <summary>
	/// Represents an Application UserPrincipal
	/// </summary>
	[AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
	public sealed class UserPrincipal : GenericPrincipal
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="UserPrincipal"></see> class from 
		/// a <see cref="UserIdentity"></see> instance and an array of role names to which
		/// the user represented by that <see cref="UserIdentity"></see> belongs.
		/// </summary>
		/// <param name="identity">An implementation of the <see cref="IIdentity"></see> interface 
		/// that represents the user.</param>
		/// <param name="roles">An array of role names to which the user represented by the 
		/// identity parameter belongs but is not used for the Silverlight portion of the application.</param>
		/// <exception cref="ArgumentNullException">identity is null.</exception>
		public UserPrincipal(UserIdentity identity, string[] roles)
			: base(identity, roles)
		{
		}
	}
	#endregion
}